﻿// PDFplacer.jsx
// An InDesign CS JavaScript
// 11 JAN 2008
// Copyright (C) 2008  Scott Zanelli. Lonelytree Software. (www.lonelytreesw.com)
// Coming to you from Quincy, MA, USA

// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


// Get app version and save old interation setting. 
// Some installs have the interaction level set to not show any dialogs.
// This is used to insure that the dialog is shown.
var appVersion = parseInt(app.version);
if(appVersion == 3){ //CS1
	var oldInteractionPref = app.userInteractionLevel;
	app.userInteractionLevel = UserInteractionLevels.interactWithAll;
}
else{ //CS2+
	var oldInteractionPref = app.scriptPreferences.userInteractionLevel;
	app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;
}

// Set default prefs
var cropType = 0;
var positionType = 4;
var offsetX = 0;
var offsetY = 0;
var doTransparent = 1;
var placeOnLayer = 0;
var fitPage = 0;
var keepProp = 0;
var addBleed = 1;

// Look for and read prefs file
prefsFile = File((Folder(app.activeScript)).parent + "/PDFplacerPrefs.txt");
if(!prefsFile.exists)
	savePrefs();
else
	readPrefs();

// Ask user to select the PDF to place
var pdfFile = File.openDialog("Select PDF file to place");

// Check name ends with PDF
if(pdfFile == null || (pdfFile.name.indexOf(".pdf") == -1 && pdfFile.name.indexOf(".PDF") == -1)){
   restoreDefaults(false);
   throwError("A PDF file must be chosen. Quitting...", false, 1, null);
}
var pdfName = File.decode(pdfFile.name);
var PDFCropTypes = [PDFCrop.cropContent, PDFCrop.cropArt, PDFCrop.cropPDF, PDFCrop.cropTrim, PDFCrop.cropBleed, PDFCrop.cropMedia];
var cropStrings = ["Bounding Box","Art","Crop","Trim","Bleed", "Media"];

// Parse the PDF file and extract needed info
var pdfInfo = getPDFInfo(pdfFile, (app.documents.length == 0));
var pdfPgCount = pdfInfo.pgCount;

// If there is no document open, create a new one using the trim size of the
// first encountered PDF page (use /TrimBox size)
if(app.documents.length == 0){
	// Save the app measurement units to restore after doc is created
	var oldUnitsV = app.viewPreferences.verticalMeasurementUnits;
	var oldUnitsH = app.viewPreferences.horizontalMeasurementUnits;
	app.viewPreferences.verticalMeasurementUnits = MeasurementUnits.points;
	app.viewPreferences.horizontalMeasurementUnits = MeasurementUnits.points;
		
	// Make the new doc:
	var theDoc = app.documents.add();
	theDoc.documentPreferences.facingPages = false;
	theDoc.marginPreferences.top = 0;
	theDoc.marginPreferences.bottom = 0;
	theDoc.marginPreferences.left = 0;
	theDoc.marginPreferences.right = 0;
	theDoc.marginPreferences.columnCount = 1;
	theDoc.documentPreferences.pageWidth = pdfInfo.pgSize.width;
	theDoc.documentPreferences.pageHeight = pdfInfo.pgSize.height;
	theDoc.viewPreferences.verticalMeasurementUnits = oldUnitsV;
	theDoc.viewPreferences.horizontalMeasurementUnits = oldUnitsH;
	
	// Restore the original units
	app.viewPreferences.verticalMeasurementUnits = oldUnitsV;
	app.viewPreferences.horizontalMeasurementUnits = oldUnitsH;
}
else{
	var theDoc = app.activeDocument;
}

if(appVersion < 5) // vers<CS3 doesn't support "activeLayer"
	var currentLayer = theDoc.layers[0];
else
	var currentLayer = theDoc.activeLayer;
var docPgCount = theDoc.pages.length;

// Loop for correct input
do{
	var dLogOK = true;
	var dLog = makeDialog();
	if(dLog.show()){
		// Extract two values from dialog info
		pdfStartPg = pdfStartPg.editValue;
		pdfEndPg = pdfEndPg.editValue;
		// Minor error check to be sure start pg is less than or equal to end pg
		if(pdfStartPg > pdfEndPg){
			alert("In the 'Place PDF pages' section, the left box must have a value less than or equal to the value in the right box. Please try again.");
			dLog.destroy();
			dLogOK = false;
		}
		else{
			// Extract the rest of the dialog info
			docStartPg = docStartPg.editValue;
			cropType = cropType.selectedIndex;
			positionType = positionType.selectedIndex;
			offsetX = offsetX.editValue;
			offsetY = offsetY.editValue;
			doTransparent = doTransparent.checkedState;
			placeOnLayer = placeOnLayer.checkedState;
			fitPage = fitPage.checkedState;
			keepProp = keepProp.checkedState;
			addBleed = addBleed.checkedState;
		}
	}
	else{
		dLog.destroy();
		restoreDefaults(false);
		exit();
	}
}while(!dLogOK);

// Dialog is no longer needed, free the memory
dLog.destroy();

// Determine if extra pages are needed and add them to end of document
var pgsToAdd = ((pdfEndPg - pdfStartPg) + 1) - ((docPgCount - docStartPg) + 1);
for(i=0; i < pgsToAdd; i++)
	theDoc.pages.add(LocationOptions.AT_END);

// Add the new layer if requested
if(placeOnLayer){
	// Add random number to file name to be layer name.
	// Double check layer name doesn't exist and alter if it happens to be present for some reason
	var layerName = pdfName + "_" + Math.round(Math.random() * 999);
	var docLayers = theDoc.layers;
	for(i=0; i < docLayers.length; i++)
	  if (docLayers[i].name.indexOf(layerName) != -1 )
		layerName += ("_" + Math.round(Math.random() * 999));
	
	// Add the layer
	currentLayer = theDoc.layers.add({name:layerName});
}

// Save zero point for later restoration
var oldZero = theDoc.zeroPoint;
// set the zero point to the origin
theDoc.zeroPoint = [0,0];

// Save ruler origin for later restoration
var oldRulerOrigin = theDoc.viewPreferences.rulerOrigin;
// set the ruler origin to page or all PDFs will be placed on first page of spreads
theDoc.viewPreferences.rulerOrigin = RulerOrigin.pageOrigin;

// Get the Indy doc's height and width
var docWidth = theDoc.documentPreferences.pageWidth;
var docHeight = theDoc.documentPreferences.pageHeight;

// Set placement prefs
with(app.pdfPlacePreferences){
	transparentBackground = doTransparent;
	pdfCrop = PDFCropTypes[cropType];
}

// Place the requested pages in the document
//*******************************************************
for(i = docStartPg - 1, currentPDFPg = pdfStartPg; currentPDFPg <= pdfEndPg; currentPDFPg++, i++){
    i++
	// Add a rectangle object to the page in which the PDF will be placed
	var theRect = theDoc.pages[i].rectangles.add(currentLayer, {fillColor: theDoc.swatches.itemByName("None"), strokeWeight: 0});

	// Set the app's PDF placement pref's page number property to the current PDF page number
	app.pdfPlacePreferences.pageNumber = currentPDFPg;

	// Place the current PDF page into the rectangle object
	try{
		theRect.place(pdfFile);
	}
	catch(e){
		if(e.name == "Error"){
			alert("\"" + pdfName + "\" doesn't contain a \"" + cropStrings[cropType] + "\" crop type:\n\nPlease try again by selecting a different crop type or open\nthe PDF in Acrobat and perform a \"Save As...\" command.", "PDF Placement Error");
		}
		else{
			alert(e);
		}
		if(placeOnLayer){
			currentLayer.remove();
		}
		else{
			theRect.remove();
		}
		restoreDefaults(true);
		exit();
	}
	
	// Fit to page size or change position
	if(fitPage){
		if(addBleed){
			// Get the bleed sizes
			var bleedT = theDoc.documentPreferences.documentBleedTopOffset;
			var bleedB = theDoc.documentPreferences.documentBleedBottomOffset;
			var bleedL = theDoc.documentPreferences.documentBleedInsideOrLeftOffset;
			var bleedR = theDoc.documentPreferences.documentBleedOutsideOrRightOffset;
			// Make rectangle the size of the page size plus bleed
			theRect.geometricBounds = [0 - bleedT, 0 - bleedL, docHeight + bleedB, docWidth + bleedR];
		}
		else{
			// Change rectangle's size to the page size
			theRect.geometricBounds = [0, 0, docHeight, docWidth];
		}
		if(keepProp)
			theRect.fit(FitOptions.proportionally);
		else
			theRect.fit(FitOptions.contentToFrame);			
	}
	else{
		// Expand frame to size of PDF
		theRect.fit(FitOptions.frameToContent);
		
		// Determine the placed PDF's dimensions
		var theRectBounds = theRect.geometricBounds; // [y1, x1, y2, x2]
		var rectW = theRectBounds[3] - theRectBounds[1];
		var rectH = theRectBounds[2] - theRectBounds[0];
		
		// Calculate edge positions
		var centeredX1 = (docWidth/2) - (rectW/2) + offsetX;
		var centeredX2 = (docWidth/2) + (rectW/2) + offsetX;
		var centeredY1 = (docHeight/2) - (rectH/2) + offsetY;
		var centeredY2 = (docHeight/2) + (rectH/2) + offsetY;
		var topY1 = 0 + offsetY;
		var topY2 = rectH + offsetY;
		var botY1 = (docHeight - rectH) + offsetY;
		var botY2 = docHeight + offsetY;
		var leftX1 = 0 + offsetX;
		var leftX2 = rectW + offsetX;
		var rightX1 = (docWidth - rectW) + offsetX;
		var rightX2 = docWidth + offsetX;
		
		// Move the placed PDF to the correct position based on user's choice
		switch(positionType){
			case 0: // Top Left
				var newPosition = [topY1, leftX1, topY2, leftX2];
				break;
			case 1: // Top Center
				var newPosition = [topY1, centeredX1, topY2, centeredX2];
				break;
			case 2: // Top Right
				var newPosition = [topY1, rightX1, topY2, rightX2];
				break;
			case 3: // Middle Left
				var newPosition = [centeredY1, leftX1, centeredY2, leftX2];
				break;
			case 4: // Center
				var newPosition = [centeredY1, centeredX1, centeredY2, centeredX2];
				break;
			case 5: // Middle Right
				var newPosition = [centeredY1, rightX1, centeredY2, rightX2];
				break;
			case 6: // Bottom Left
				var newPosition = [botY1, leftX1, botY2, leftX2];
				break;
			case 7: // Bottom Center
				var newPosition = [botY1, centeredX1, botY2, centeredX2];
				break;
			case 8: // Bottom Right
				var newPosition = [botY1, rightX1, botY2, rightX2];
				break;
		}		
		theRect.geometricBounds = newPosition;
		theRect.allGraphics[0].geometricBounds = newPosition;
	}
}

// Save prefs and then restore original app/doc settings
savePrefs();
restoreDefaults(true);

// THE END OF EXECUTION

// Function that creates the dialog box
function makeDialog(){
	// These make following lines shorter and more readable
	var intPropertiesS = {editValue:1, largeNudge:10, smallNudge:1,minimumValue:1,maximumValue:pdfPgCount};
	var intPropertiesE = {editValue:pdfPgCount, largeNudge:10, smallNudge:1,minimumValue:1,maximumValue:pdfPgCount};
	var intPropertiesA = {editValue:1, largeNudge:10, smallNudge:1,minimumValue:1,maximumValue:docPgCount};
	var realPropertiesX = {editValue:offsetX, largeNudge:0.1, smallNudge:0.001};
	var realPropertiesY = {editValue:offsetY, largeNudge:0.1, smallNudge:0.001};
	var positionStrings = ["Top Left", "Top Center", "Top Right", "Middle Left", "Center", "Middle Right", "Bottom Left", "Bottom Center", "Bottom Right"];

	var dLog = app.dialogs.add({name:"Place Multiple PDF Pages", canCancel:true});
	with(dLog){
		// left column
		with(dialogColumns.add()){
			with (dialogRows.add())
				staticTexts.add({staticLabel:"Page Selection"});
			with(borderPanels.add()){
				with (dialogColumns.add()){
					with (dialogRows.add())
						staticTexts.add({staticLabel:"Place PDF Pages:"});
					with (dialogRows.add()){
						pdfStartPg = integerEditboxes.add(intPropertiesS);
						staticTexts.add({staticLabel:"thru"});
						pdfEndPg = integerEditboxes.add(intPropertiesE);
					}
					with (dialogRows.add())
						staticTexts.add({staticLabel:"Start Placing on Page:"});
					with (dialogRows.add())
						docStartPg = integerEditboxes.add(intPropertiesA);
				}
			}
			with (dialogRows.add())
				staticTexts.add({staticLabel:"PDF Sizing Options"});
			with(borderPanels.add()){
				with (dialogColumns.add()){
					with(dialogRows.add())
						fitPage = checkboxControls.add({staticLabel:"Fit PDF to Page", checkedState:fitPage});
					with(dialogRows.add())
						keepProp = checkboxControls.add({staticLabel:"Keep Proportions", checkedState:keepProp});
					with(dialogRows.add())
						addBleed = checkboxControls.add({staticLabel:"Bleed the Fit Page", checkedState:addBleed, minWidth:174});			
				}
			}
		}
		// right column
		with(dialogColumns.add()){
			with (dialogRows.add())
				staticTexts.add({staticLabel:"Positioning Options"});
			with(borderPanels.add()){
				with (dialogColumns.add()){
					with (dialogRows.add())
						staticTexts.add({staticLabel:"Position PDFs on Page Aligned From:"});
					with(dialogRows.add())
						positionType = dropdowns.add({stringList:positionStrings, selectedIndex:positionType, minWidth:195});
					with (dialogRows.add())
						staticTexts.add({staticLabel:"Offset by:"});
					with (dialogRows.add()){
						staticTexts.add({staticLabel:"X:"});
						offsetX = realEditboxes.add(realPropertiesX);
						staticTexts.add({staticLabel:"Y:"});
						offsetY = realEditboxes.add(realPropertiesY);
					}
				}
			}
			with (dialogRows.add())
				staticTexts.add({staticLabel:"Placement Options"});
			with(borderPanels.add()){
				with (dialogColumns.add()){
					with(dialogRows.add()){
						staticTexts.add({staticLabel:"Crop to:"});
						cropType = dropdowns.add({stringList:cropStrings, selectedIndex:cropType, minWidth:156});//
					}
					with (dialogRows.add())
						doTransparent = checkboxControls.add({staticLabel:"Transparent Background", checkedState:doTransparent, minWidth:203});
					with (dialogRows.add())
						placeOnLayer = checkboxControls.add({staticLabel:"Place PDF on a New Layer", checkedState:placeOnLayer});
				}
			}
		}
	}
	return dLog;
}

// function to restore saved settings back to originals before script ran
// extras parameter is for exiting at different areas of script:
// 1: prior to doing anything
// 2: end of script
// 3: reading PDF file size
function restoreDefaults(extras){
	if(appVersion == 3){ //CS1
		app.userInteractionLevel = oldInteractionPref;
	}
	else{ //CS2+
		app.scriptPreferences.userInteractionLevel = oldInteractionPref;
	}
	if(extras == true){
		theDoc.zeroPoint = oldZero;
		theDoc.viewPreferences.rulerOrigin = oldRulerOrigin;
	}
}

// function to read prefs from a file
function readPrefs(){
	prefsFile.open("r");
	cropType = Number(prefsFile.readln() );
	positionType = Number(prefsFile.readln() );
	offsetX = Number(prefsFile.readln() );
	offsetY = Number(prefsFile.readln() );
	doTransparent = Number(prefsFile.readln() );
	placeOnLayer = Number(prefsFile.readln() );
	fitPage = Number(prefsFile.readln() );
	keepProp = Number(prefsFile.readln() );
	addBleed = Number(prefsFile.readln() );
	prefsFile.close();
}

// function to save prefs to a file
function savePrefs(){
	var newPrefs =
	cropType + "\n" +
	positionType + "\n" +
	offsetX + "\n" +
	offsetY + "\n" +
	((doTransparent)?1:0) + "\n" +
	((placeOnLayer)?1:0) + "\n" +
	((fitPage)?1:0) + "\n" +
	((keepProp)?1:0) + "\n" +
	((addBleed)?1:0) + "\n" +
	"0\n0\n0\n0\n0\n0\n0\n0\n0\n0\n";// Add (10) for extra prefs in future
	prefsFile.open("w");
	prefsFile.write(newPrefs);
	prefsFile.close();
}


/*********************************************/
/*                                           */
/*           PDF READER SECTION              */
/*     Extracts count and size of pages      */
/*                                           */
/*********************************************/

// Extract info from the PDF file.
// getSize is a boolean that will also determine page size and rotation of first page
// *** File position changes in this function. ***
// Results are as follows:
// page count = retArray.pgCount
// page width = retArray.pgSize.pgWidth
// page height = retArray.pgSize.pgHeight
function getPDFInfo(pdfFile, getSize){
	var flag = 0; // used to keep track if the %EOF line was encountered
	var nlCount = 0; // number of newline characters per line (1 or 2)
	
	// The array to hold return values
	var retArray = new Array();
	retArray["pgCount"] = -1;
	retArray["pgSize"] = null;
	
	// Open the PDF file for reading
	pdfFile.open("r");
		
	// Search for %EOF line
	// This skips any garbage at the end of the file
	// if FOE% is encountered (%EOF read backwards), flag will be 15 
	for(i=0; flag != 15; i++){
		pdfFile.seek(i,2);
		switch(pdfFile.readch()){
			case "F":
				flag|=1;
				break;
			case "O":
				flag|=2;
				break;
			case "E":
				flag|=4;
				break;
			case "%":
				flag|=8;
				break;
			default:
				flag=0;
				break;
		}	
	}
	// Jump back a small distance to allow going forward more easily
	pdfFile.seek(pdfFile.tell()-100);
	
	// Read until startxref section is reached
	while(pdfFile.readln() != "startxref");
	
	// Set the position of the first xref section
	var xrefPos = parseInt(pdfFile.readln(), 10);

	// The array for all the xref sections
	var	xrefArray = new Array();
	
	// Go to the xref section
	pdfFile.seek(xrefPos);
	
	// Determine length of xref entries
	// (not all PDFs are compliant with the requirement of 20 char/entry)
	xrefArray["lineLen"] = determineLineLen(pdfFile);
	
	// Get all the xref sections
	while(xrefPos != -1){
		// Go to next section
		pdfFile.seek(xrefPos);
		
		// Make sure it's an xref line we went to, otherwise PDF is no good
		if (pdfFile.readln() != "xref"){
			throwError("Cannot determine page count.", true, 99, pdfFile);
		}		
	
		// Add the current xref section into the main array
		xrefArray[xrefArray.length] = makeXrefEntry(pdfFile, xrefArray.lineLen);
		
		// See if there are any more xref sections
		xrefPos = xrefArray[xrefArray.length-1].prevXref;
	}
	
	// Go get the location of the /Catalog section (the /Root obj)
	var objRef = -1; 
	for(i=0; i < xrefArray.length; i++){
		objRef = xrefArray[i].rootObj;
		if(objRef != -1){
			i = xrefArray.length;
		}
	}
	
	// Double check root obj was found
	if(objRef == -1){
		throwError("Unable to find Root object.", true, 98, pdfFile);
	}
	
	// Get the offset of the root section and set file position to it
	var theOffset = getByteOffset(objRef, xrefArray);
	pdfFile.seek(theOffset);
	
	// Determine the obj where the first page is located
	objRef = getRootPageNode(pdfFile);
	
	// Get the offset where the root page nod is located and set the file position to it
	theOffset = getByteOffset(objRef, xrefArray);
	pdfFile.seek(theOffset);

	// Get the page count info from the root page tree node section
	retArray.pgCount = readPageCount(pdfFile);
	
	// Does user need size also? If so, get size info
	if(getSize){
		// Go back to root page tree node
		pdfFile.seek(theOffset);
		
		// Flag to tell if page tree root was visited already
		var rootFlag = false;

		// Loop until an actual page obj is found (page tree leaf)
		do{
			var getOut = true;
			
			if(rootFlag){
				// Try to find the line with the /Kids entry
				// Also look for instance when MediBox is in the root obj
				do{
					var tempLine = pdfFile.readln();
				}while(tempLine.indexOf("/Kids") == -1 && tempLine.indexOf(">>") == -1);
				
			}
			else{
				// Try to first find the line with the /MediaBox entry
				rootFlag = true; // Indicate root page tree was visited
				getOut = false; // Force loop if /MediaBox isn't found here
				do{
					var tempLine = pdfFile.readln();
					if(tempLine.indexOf("/MediaBox") != -1){
						getOut = true;
						break;
					}
				}while(tempLine.indexOf(">>") == -1);
				
				if(!getOut){
					// Reset the file pointer to the beginning of the root obj again
					pdfFile.seek(theOffset)
				}
			}
			
			// If /Kids entry was found, still at an internal page tree node
			if(tempLine.indexOf("/Kids") != -1){
				// Check if the array is on the same line
				if(tempLine.indexOf("R") != -1){
					// Grab the obj ref for the first page
					objRef = parseInt(tempLine.split("/Kids")[1].split("[")[1]);
				}
				else{
					// Go down one line
					tempLine = pdfFile.readln();
					
					// Check if the opening bracket is on this line
					if(tempLine.indexOf("[") != -1){
						// Grab the obj ref for the first page
						objRef = parseInt(tempLine.split("[")[1]);
					}
					else{
						// Grab the obj ref for the first page
						objRef = parseInt(tempLine);
					}
				
				}
		
				// Get the file offset for the page obj and set file pos to it
				theOffset = getByteOffset(objRef, xrefArray);
				pdfFile.seek(theOffset);
				getOut = false;
			}
		}while(!getOut);
		
		// Make sure file position is correct if finally at a leaf
		pdfFile.seek(theOffset);
		
		// Go get the page sizes
		retArray.pgSize = getPageSize(pdfFile);		
	}	
		
	// Close the PDF file, finally all done!
	pdfFile.close();
	
	return retArray;
}

// Function to create an array of xref info
// File position must be set to second line of xref section
// *** File position changes in this function. ***
function makeXrefEntry(pdfFile, lineLen){
	var newEntry = new Array();
	newEntry["theSects"] = new Array();
	var temp = pdfFile.readln();
	
	// Save info
	newEntry.theSects[0] = makeXrefSection(temp, pdfFile.tell());
		
	// Try to get to trailer line
	var xrefSec = newEntry.theSects[newEntry.theSects.length-1].refPos;
	var numObjs = newEntry.theSects[newEntry.theSects.length-1].numObjs;
	do{
		var getOut = true;
		pdfFile.seek(xrefSec  + (numObjs * lineLen));
		var tempLine = pdfFile.readln();
		if(tempLine.indexOf("trailer") == -1){
			// Found another xref section, create an entry for it
			var tempArray = makeXrefSection(tempLine, pdfFile.tell());
			newEntry.theSects[newEntry.theSects.length] = tempArray;
			xrefSec = tempArray.refPos;
			numObjs = tempArray.numObjs;
			getOut = false;
		}
	}while(!getOut);
	
	// Read line with trailer dict info in it
	// Need to get /Root object ref
	newEntry["rootObj"] = -1;
	newEntry["prevXref"] = -1;
	do{
		temp = pdfFile.readln();
		if(temp.indexOf("/Root") != -1){
			// Extract the obj location where the root of the page tree is located:
			newEntry.rootObj = parseInt(temp.substring(temp.indexOf("/Root") + 5), 10);
		}
		if(temp.indexOf("/Prev") != -1){
			newEntry.prevXref = parseInt(temp.substring(temp.indexOf("/Prev") + 5), 10);		
		}
			
	}while(temp.indexOf(">>") == -1);
	
	return newEntry;
}

// Function to save xref info to a given array
function makeXrefSection(theLine, thePos){
	var tempArray = new Array();
	var temp = theLine.split(" ");
	tempArray["startObj"] = parseInt(temp[0], 10);
	tempArray["numObjs"] = parseInt(temp[1], 10);
	tempArray["refPos"] = thePos;
	return tempArray;
}

// Function that gets the page count form a root page section
// *** File position changes in this function. ***
function readPageCount(pdfFile){
	// Read in first line of section
	var theLine = pdfFile.readln();
	
	// Locate the line containing the /Count entry
	while(theLine.indexOf("/Count") == -1){ // если строка не содержит данной подстроки, то читаем следующую строку
		
		theLine = pdfFile.readln();
	}
	
	// Extract the page count
	return parseInt(theLine.substring(theLine.indexOf("/Count") +6), 10);
}

// Function to determine length of xref entries
// Not all PDFs conform to the 20 char/entry requirement
// *** File position changes in this function. ***
function determineLineLen(pdfFile){
	// Skip xref line
	pdfFile.readln();
	var lineLen = -1;

	// Loop trying to find lineLen
	do{
		var getOut = true;
		 var tempLine = pdfFile.readln();
		if(tempLine != "trailer"){
			// Get the number of object enteries in this section
			var numObj = parseInt(tempLine.split(" ")[1]);
			
			// If there is more than one entry in this section, use them to determime lineLen
			if(numObj > 1){
				var tempPos = pdfFile.tell();
				pdfFile.readln();
				lineLen = pdfFile.tell() - tempPos;
			}
			else{
				if(numObj ==1){
					// Skip the single entry
					pdfFile.readln();
				}
				getOut = false;
			}
		}
		else{
			// Read next line(s) and extract previous xref section
			var getOut = false;
			do{
				tempLine = pdfFile.readln();
				if(tempLine.indexOf("/Prev") != -1){
					pdfFile.seek(parseInt(tempLine.substring(tempLine.indexOf("/Prev") + 5)));
					getOut = true;
				}
			}while(tempLine.indexOf(">>") == -1 && !getOut);
			pdfFile.readln(); // Skip the xref line
			getOut = false;
		}
	}while(!getOut);
	
	// Check if there was a problem determining the line length
	if(lineLen == -1){
		throwError("Unable to determine xref dictionary line length.", true, 97, pdfFile);
	}

	return lineLen;
}

// Function that determines the byte offset of an object number
// Searches the built array of xref sections and reads the offset for theObj
// *** File position changes in this function. ***
function getByteOffset(theObj, xrefArray){
	var theOffset = -1;
	
	// Look for the theObj in all sections found previously
	for(i = 0; i < xrefArray.length; i++){
		var tempArray = xrefArray[i];
		for(j=0; j < tempArray.theSects.length; j++){
			 var tempArray2 = tempArray.theSects[j]; 
			
			// See if theObj falls within this section
			if(tempArray2.startObj <= theObj && theObj <= tempArray2.startObj + tempArray2.numObjs -1){
				pdfFile.seek((tempArray2.refPos + ((theObj - tempArray2.startObj) * xrefArray.lineLen)));
				
				// Get the location of the obj
				var tempLine = pdfFile.readln();
				
				// Check if this is an old obj, if so ignore it
				// An xref entry with n is live, with f is not
				if(tempLine.indexOf("n") != -1){
					theOffset = parseInt(tempLine, 10);
				
					// Cleanly get out of both loops
					j = tempArray.theSects.length;
					i = xrefArray.length;
				}
			}
		}
	}

	return theOffset;
}

// Function to extract the root page node object from a section
// File position must be at the start of the root page node
// *** File position changes in this function. ***
function getRootPageNode(pdfFile){
	var tempLine = pdfFile.readln();
	
	// Go to line with /Page token in it
	while(tempLine.indexOf("/Pages") == -1){
		tempLine = pdfFile.readln();
	}

	// Extract the root page obj number
	return parseInt(tempLine.substring(tempLine.indexOf("/Pages") + 6), 10);
}

// Function to extract the sizes from a page reference section
// File position must be at the start of the page object
// *** File position changes in this function. ***
function getPageSize(pdfFile){
	var hasTrimBox = false; // Prevent MediaBox from overwriting TrimBox info
	var charOffset = -1;
	var isRotated = false; // Page rotated 90 or 270 degrees?
	var foundSize = false; // Was a size found?
	do{
		var theLine = pdfFile.readln();
		if(!hasTrimBox && (charOffset = theLine.indexOf("/MediaBox")) != -1){
			// Is the array on the same line?
			if(theLine.indexOf("[", charOffset + 9) == -1){
				// Need to go down one line to find the array
				theLine = pdfFile.readln();
				// Extract the values of the MediaBox array (x1, y1, x2, y2)
				var theNums = theLine.split("[")[1].split("]")[0].split(" ");
			}
			else{
				// Extract the values of the MediaBox array (x1, y1, x2, y2)
				var theNums = theLine.split("/MediaBox")[1].split("[")[1].split("]")[0].split(" ");
			}
			
			// Take care of leading space
			if(theNums[0] == ""){
				theNums = theNums.slice(1);
			}
			
			foundSize = true;
		}
		if((charOffset = theLine.indexOf("/TrimBox")) != -1){
			// Is the array on the same line?
			if(theLine.indexOf("[", charOffset + 8) == -1){
				// Need to go down one line to find the array
				theLine = pdfFile.readln();
				// Extract the values of the MediaBox array (x1, y1, x2, y2)
				var theNums = theLine.split("[")[1].split("]")[0].split(" ");
			}
			else{
				// Extract the values of the MediaBox array (x1, y1, x2, y2)
				var theNums = theLine.split("/TrimBox")[1].split("[")[1].split("]")[0].split(" ");
			}
	
			// Prevent MediaBox overwriting TrimBox values
			hasTrimBox = true;
			
			// Take care of leading space
			if(theNums[0] == ""){
				theNums = theNums.slice(1);
			}

			foundSize = true;
		}
		if((charOffset = theLine.indexOf("/Rotate") ) != 1){
			var rotVal = parseInt(theLine.substring(charOffset + 7));
			if(rotVal == 90 || rotVal == 270){
				isRotated = true;
			}
		}
	}while(theLine.indexOf(">>") == -1);
	
	// Check if a size array wasn't found
	if(!foundSize){
		throwError("Unable to determine PDF page size.", true, 96, pdfFile);
	}
	
	// Do the math
	var xSize =  parseFloat(theNums[2]) - parseFloat(theNums[0]);
	var ySize =  parseFloat(theNums[3]) - parseFloat(theNums[1]);
	
	// One last check that sizes are actually numbers
	if(isNaN(xSize) || isNaN(ySize)){
		throwError("One or both page dimensions could not be calculated.", true, 95, pdfFile);
	}
	
	// Use rotation to determine orientation of pages
	var ret = new Array();
	ret["width"] = isRotated ? ySize : xSize;
	ret["height"] = isRotated ? xSize : ySize;

	return ret;
}

// Error function
function throwError(msg, includeResave, idNum, fileToClose){
	var RESAVE = "\n\nPlease open the PDF file in Acrobat, do a \"Save As...\" and try again.";
	alert("ERROR: " + msg + ((includeResave) ? RESAVE : "") + " (" + idNum + ")", "PDF Placer Script Error");
	if(fileToClose != null){
		fileToClose.close();
	}
	exit(idNum);
}
